-- --8<--8<--8<--8<--
--
-- Copyright (C) 2007 Smithsonian Astrophysical Observatory
--
-- This file is part of AXAFCoords_lua
--
-- AXAFCoords_lua is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or (at
-- your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- -->8-->8-->8-->8--

local math = require( "math" )

module( ... )

local pi, acos, cos, mod, atan2, sin, tan, asin, atan=
   math.pi, math.acos, math.cos, math.mod,
   math.atan2, math.sin, math.tan, math.asin, math.atan

sec2rad = pi / ( 180 * 3600 )
min2rad = pi / ( 180 * 60 )
deg2rad = pi / 180
rad2deg = 180 / pi

-- el and az are in radians
function osac_elaz_2_osac_polar( el, az )

  local theta, phi

  el = mod( el, pi/2 )
  az = mod( az, pi/2 )

  -- since -pi/2 <= el <= pi/2 and -pi/2 <= az <= pi/2,
  -- cos(el) >= 0 and cos(az) >= 0, so theta >= 0
  theta = acos( cos(el * rad2deg ) * cos( az * rad2deg ) ) * deg2rad

  -- atan2 returns -pi <= phi <= pi,
  -- tan(el) and sin(az) preserve the signs of el and az.
  phi = atan2( tan(el * rad2deg ), sin(az * rad2deg ) ) * deg2rad

  -- convert phi s.t. 0 <= phi < 2*pi
  phi = mod( phi + 2 * pi, 2 * pi );

  return theta, phi

end

-- theta and phi are in radians
-- returns el and az in radians
function osac_polar_2_osac_elaz( theta, phi )

  local el, az

  -- convert phi to 0 <= phi < 2*pi
  phi   = mod( mod( phi,   2 * pi ) + 2 * pi, 2 * pi)
  theta = mod( theta, pi / 2 ) 

  -- theta is always < $pi/2, so sin( theta ) > 0
  -- sign of sin(el) tracks sin(phi)
  el = asin( sin( theta * rad2deg ) * sin( phi *rad2deg ) ) * deg2rad

  -- theta is always < $pi/2, so tan( theta ) > 0
  -- sign of az tracks cos(phi).
  az = atan( cos( phi * rad2deg ) * tan ( theta * rad2deg ) ) * deg2rad

  return el, az
end

-- theta and phi are in radians
-- raygen polar is off by pi
function osac_polar_2_raygen_polar( theta, phi )

  phi = mod( phi + pi, 2 * pi )

  return theta, phi
end

-- convert from osac_polar to HRMA Left Handed Spherical Coordinates
-- ( AXAF-HSC-1.1 )
function osac_polar_2_HSC11( theta, phi )

  phi = mod( phi + pi, 2 * pi )

  return theta, phi
end

-- convert from HRMA Left Handed Spherical Coordinates
-- ( AXAF-HSC-1.1 ) to osac_polar 
function HSC11_2_osac_polar( theta, phi )

  phi = mod( phi + pi, 2 * pi )

  return theta, phi
end

-- convert from XRCF pitch/yaw to OSAC el/az
function pitchyaw_2_osac_elaz( pitch, yaw )

  return (- pitch * min2rad), (- yaw * min2rad)

end

-- convert from raygen_polar to Mirror Spherical Coordinates
function raygen_polar_to_MSC( theta, phi )

  phi = mod( phi + pi, 2 * pi )

  return theta, phi
end

-- convert from Mirror Spherical Coordinates to raygen_polar 
function MSC_to_raygen_polar( theta, phi )

  phi = mod( phi + pi, 2 * pi )

  return theta, phi
end


-- convert from OSAC elevation and azimuth to X and Y at a particular Z
function osac_elaz_2_xy( el, az, z )

  local theta, phi = osac_elaz_2_osac_polar( el, az )
  phi = phi * rad2deg
  theta = theta * rad2deg

  local r = z * sin( theta )
  local y = r * sin( phi )
  local x = r * cos( phi )

  return x, y
end


-- Convert from B<raygen> polar coordinates to B<raygen> elevation and
-- azimuth.  All angles are in radians.  Input angles are reduced such
-- that 0 <= phi < 2pi and 0 <= theta <= pi/2

function raygen_polar_2_raygen_elaz( theta, phi )

  local el, az

  phi   = phi   * rad2deg
  theta = theta * rad2deg

  -- convert phi to 0 <= phi < 2*pi, theta to 0 <= theta < pi/2
  phi   = mod( mod( phi, 360 ) + 360, 360)
  theta = mod( theta, 90 )

  -- theta is always < pi/2, so sin( theta ) > 0
  -- sign of sin(el) tracks sin(phi)
  el = asin( sin( theta ) * sin( phi ) )

  -- theta is always < pi/2, so tan( theta ) > 0
  -- sign of az tracks cos(phi).
  az = atan( cos( phi) * tan ( theta ) )

  return el * deg2rad, az * deg2rad
end



-- Convert from raygen elevation and azimuth to raygen polar coordinates.
-- All angles are in radians.  Input angles are reduced to be 0 <= angle
-- <= pi/2.

function raygen_elaz_2_raygen_polar( el, az )
  local theta, phi

  el = el * rad2deg
  az = az * rad2deg

  el = mod( el, 360 )
  az = mod( az, 360 )

  -- since -pi/2 <= el <= pi/2 and -pi/2 <= az <= pi/2,
  -- cos(el) >= 0 and cos(az) >= 0, so theta >= 0
  theta = acos( cos(el) * cos(az) )

  -- atan2 returns -pi <= phi <= pi,
  -- tan(el) and sin(az) preserve the signs of el and az.
  phi = atan2( tan(el), sin(az) )

  -- convert phi s.t. 0 <= phi < 2*pi
  phi   = mod( phi + 360, 360 )

  return theta * deg2rad, phi * deg2rad

end
